/* Includes *****************************************************************/

#include "api_amd.h"
#include "debug.h"

/* Constants ****************************************************************/

const char *ADAPTER_AMD = "AMD";

/* Functions ****************************************************************/

HMODULE load_library(const char *vendor, const char *name)
{
	HMODULE library = LoadLibraryA(name);

	if (!library)
	{
		debug_printf("LOAD(%s): LoadLibrary(\"%s\") = ERROR %i", vendor, name, GetLastError());
		return NULL;
	}

	debug_printf("LOAD(%s): LoadLibrary(\"%s\") = SUCCESS", vendor, name);
	return library;
}

/****************************************************************************/

BOOL load_function(HMODULE library, const char *vendor, void **function, const char *name)
{
	*function = GetProcAddress(library, name);

	if (!function)
	{
		debug_printf("LOAD(%s): GetProcAddress(\"%s\") = ERROR %i", vendor, name, GetLastError());
		return FALSE;
	}

	//debug_printf("LOAD(%s): GetProcAddress(\"%s\") = SUCCESS", vendor, name);
	return TRUE;
}

/****************************************************************************/

void *CALLBACK ADL_Main_Memory_Alloc(int size)
{
	return malloc(size);
}

/****************************************************************************/

BOOL load_amd()
{
	HMODULE library;
	int status;

	library = load_library(ADAPTER_AMD, "atiadlxx.dll");

	if (!library)
	{
		library = load_library(ADAPTER_AMD, "atiadlxy.dll");

		if (!library)
			return FALSE;
	}

	if (!load_function(library, ADAPTER_AMD, (void **)&ADL_Main_Control_Create, "ADL_Main_Control_Create"))
		return FALSE;

	if (!load_function(library, ADAPTER_AMD, (void **)&ADL_Main_Control_Refresh, "ADL_Main_Control_Refresh"))
		return FALSE;

	if (!load_function(library, ADAPTER_AMD, (void **)&ADL_Adapter_NumberOfAdapters_Get, "ADL_Adapter_NumberOfAdapters_Get"))
		return FALSE;

	if (!load_function(library, ADAPTER_AMD, (void **)&ADL_Adapter_AdapterInfo_Get, "ADL_Adapter_AdapterInfo_Get"))
		return FALSE;

	if (!load_function(library, ADAPTER_AMD, (void **)&ADL_Display_DisplayInfo_Get, "ADL_Display_DisplayInfo_Get"))
		return FALSE;

	if (!load_function(library, ADAPTER_AMD, (void **)&ADL_Display_DDCBlockAccess_Get, "ADL_Display_DDCBlockAccess_Get"))
		return FALSE;

	status = ADL_Main_Control_Create(ADL_Main_Memory_Alloc, 1);

	if (status != ADL_OK)
	{
		debug_printf("LOAD(%s): ADL_Main_Control_Create() = ERROR %i", ADAPTER_AMD, status);
		return FALSE;
	}

	debug_printf("LOAD(%s): ADL_Main_Control_Create() = SUCCESS", ADAPTER_AMD);
	return TRUE;
}

/****************************************************************************/

BOOL refresh_amd()
{
	int status;

	status = ADL_Main_Control_Refresh();

	if (status != ADL_OK)
	{
		debug_printf("LOAD(%s): ADL_Main_Control_Refresh() = ERROR %i", ADAPTER_AMD, status);
		return FALSE;
	}

	debug_printf("LOAD(%s): ADL_Main_Control_Refresh() = SUCCESS", ADAPTER_AMD);
	return TRUE;
}

/****************************************************************************/
