/* Includes *****************************************************************/

#include "debug.h"

/* Constants ****************************************************************/

static const char *DEBUG_FILE = "debug.txt";

/* Locals *******************************************************************/

static int debug_level; // 0 = disabled, 1 = reads only, 2 = reads and writes

/* Functions ****************************************************************/

BOOL set_debug_level(int new_debug_level)
{
	debug_level = new_debug_level;
	return TRUE;
}

/****************************************************************************/

int get_debug_level()
{
	return debug_level;
}

/****************************************************************************/

FILE *debug_file()
{
	static FILE *file;

	if (!file)
	{
		file = fopen(DEBUG_FILE, "a");

		if (!file)
			return NULL;

		fseek(file, 0, SEEK_END);

		if (ftell(file) > 0)
			fprintf(file, "\n");
	}

	return file;
}

/****************************************************************************/

BOOL debug_timestamp()
{
	SYSTEMTIME time;

	GetLocalTime(&time);
	fprintf(debug_file(), "[%i-%02i-%02i %02i:%02i:%02i.%03i] ", time.wYear, time.wMonth, time.wDay, time.wHour, time.wMinute, time.wSecond, time.wMilliseconds);
	return TRUE;
}

/****************************************************************************/

BOOL debug_printf(const char *format, ...)
{
	va_list list;

	if (get_debug_level() == 0)
		return TRUE;

	if (!debug_file())
		return FALSE;

	debug_timestamp();
	va_start(list, format);
	vfprintf(debug_file(), format, list);
	va_end(list);
	fprintf(debug_file(), "\n");
	fflush(debug_file());
	return TRUE;
}

/****************************************************************************/
