/* Includes *****************************************************************/

#include "hotkeys.h"

/* Constants ****************************************************************/

static const UINT default_modifiers = HK_ALT | HK_CTRL;

/****************************************************************************/

static struct
{
	const UINT id;
	const UINT vkey;
	const BOOL extended;
	const LPCTSTR text;
	const UINT set;
}
hotkey_list[] =
{
	// Identifier			Virtual Key		Extended Key	Text			Key Set
	{IDM_STROBING_ON,		VK_OEM_PLUS,	FALSE,			_T("+"),		HKS_STROBING},
	{IDM_STROBING_OFF,		VK_OEM_MINUS,	FALSE,			_T("-"),		HKS_STROBING},

	{IDM_STROBING_ON,		VK_ADD,			FALSE,			_T("+"),		HKS_STROBING_NUMPAD},
	{IDM_STROBING_OFF,		VK_SUBTRACT,	FALSE,			_T("-"),		HKS_STROBING_NUMPAD},

	{IDM_BRIGHTNESS_10,		'1',			FALSE,			_T("1"),		HKS_BRIGHTNESS},
	{IDM_BRIGHTNESS_20,		'2',			FALSE,			_T("2"),		HKS_BRIGHTNESS},
	{IDM_BRIGHTNESS_30,		'3',			FALSE,			_T("3"),		HKS_BRIGHTNESS},
	{IDM_BRIGHTNESS_40,		'4',			FALSE,			_T("4"),		HKS_BRIGHTNESS},
	{IDM_BRIGHTNESS_50,		'5',			FALSE,			_T("5"),		HKS_BRIGHTNESS},
	{IDM_BRIGHTNESS_60,		'6',			FALSE,			_T("6"),		HKS_BRIGHTNESS},
	{IDM_BRIGHTNESS_70,		'7',			FALSE,			_T("7"),		HKS_BRIGHTNESS},
	{IDM_BRIGHTNESS_80,		'8',			FALSE,			_T("8"),		HKS_BRIGHTNESS},
	{IDM_BRIGHTNESS_90,		'9',			FALSE,			_T("9"),		HKS_BRIGHTNESS},
	{IDM_BRIGHTNESS_100,	'0',			FALSE,			_T("0"),		HKS_BRIGHTNESS},

	{IDM_BRIGHTNESS_10,		VK_NUMPAD1,		FALSE,			_T("1"),		HKS_BRIGHTNESS_NUMLOCK_ON},
	{IDM_BRIGHTNESS_20,		VK_NUMPAD2,		FALSE,			_T("2"),		HKS_BRIGHTNESS_NUMLOCK_ON},
	{IDM_BRIGHTNESS_30,		VK_NUMPAD3,		FALSE,			_T("3"),		HKS_BRIGHTNESS_NUMLOCK_ON},
	{IDM_BRIGHTNESS_40,		VK_NUMPAD4,		FALSE,			_T("4"),		HKS_BRIGHTNESS_NUMLOCK_ON},
	{IDM_BRIGHTNESS_50,		VK_NUMPAD5,		FALSE,			_T("5"),		HKS_BRIGHTNESS_NUMLOCK_ON},
	{IDM_BRIGHTNESS_60,		VK_NUMPAD6,		FALSE,			_T("6"),		HKS_BRIGHTNESS_NUMLOCK_ON},
	{IDM_BRIGHTNESS_70,		VK_NUMPAD7,		FALSE,			_T("7"),		HKS_BRIGHTNESS_NUMLOCK_ON},
	{IDM_BRIGHTNESS_80,		VK_NUMPAD8,		FALSE,			_T("8"),		HKS_BRIGHTNESS_NUMLOCK_ON},
	{IDM_BRIGHTNESS_90,		VK_NUMPAD9,		FALSE,			_T("9"),		HKS_BRIGHTNESS_NUMLOCK_ON},
	{IDM_BRIGHTNESS_100,	VK_NUMPAD0,		FALSE,			_T("0"),		HKS_BRIGHTNESS_NUMLOCK_ON},

	{IDM_BRIGHTNESS_10,		VK_END,			FALSE,			_T("1"),		HKS_BRIGHTNESS_NUMLOCK_OFF},
	{IDM_BRIGHTNESS_20,		VK_DOWN,		FALSE,			_T("2"),		HKS_BRIGHTNESS_NUMLOCK_OFF},
	{IDM_BRIGHTNESS_30,		VK_NEXT,		FALSE,			_T("3"),		HKS_BRIGHTNESS_NUMLOCK_OFF},
	{IDM_BRIGHTNESS_40,		VK_LEFT,		FALSE,			_T("4"),		HKS_BRIGHTNESS_NUMLOCK_OFF},
	{IDM_BRIGHTNESS_50,		VK_CLEAR,		FALSE,			_T("5"),		HKS_BRIGHTNESS_NUMLOCK_OFF},
	{IDM_BRIGHTNESS_60,		VK_RIGHT,		FALSE,			_T("6"),		HKS_BRIGHTNESS_NUMLOCK_OFF},
	{IDM_BRIGHTNESS_70,		VK_HOME,		FALSE,			_T("7"),		HKS_BRIGHTNESS_NUMLOCK_OFF},
	{IDM_BRIGHTNESS_80,		VK_UP,			FALSE,			_T("8"),		HKS_BRIGHTNESS_NUMLOCK_OFF},
	{IDM_BRIGHTNESS_90,		VK_PRIOR,		FALSE,			_T("9"),		HKS_BRIGHTNESS_NUMLOCK_OFF},
	{IDM_BRIGHTNESS_100,	VK_INSERT,		FALSE,			_T("0"),		HKS_BRIGHTNESS_NUMLOCK_OFF},

	{IDM_120_STROBED,		VK_F12,			FALSE,			_T("F12"),		HKS_FUNCTION},
	{IDM_110_STROBED,		VK_F11,			FALSE,			_T("F11"),		HKS_FUNCTION},
	{IDM_100_STROBED,		VK_F10,			FALSE,			_T("F10"),		HKS_FUNCTION},
	{IDM_60,				VK_F6,			FALSE,			_T("F6"),		HKS_FUNCTION},

	{0}
};

/****************************************************************************/

static struct
{
	const UINT modifier;
	const UINT left_modifier;
	const UINT right_modifier;
	const LPCTSTR text;
	const LPCTSTR left_text;
	const LPCTSTR right_text;
}
modifier_list[] =
{
	{HK_CTRL,	HK_LCTRL,	HK_RCTRL,		_T("Ctrl"),		_T("LCtrl"),	_T("RCtrl")},
	{HK_ALT,	HK_LALT,	HK_RALT,		_T("Alt"),		_T("LAlt"),		_T("RAlt")},
	{HK_SHIFT,	HK_LSHIFT,	HK_RSHIFT,		_T("Shift"),	_T("LShift"),	_T("RShift")},
	{HK_WIN,	HK_LWIN,	HK_RWIN,		_T("Win"),		_T("LWin"),		_T("RWin")},
	{0}
};

/* Locals *******************************************************************/

static UINT disabled_hotkeys;
static UINT hotkey_modifiers;
static UINT modifier_state;
static BOOL key_state[256];
static HWND hotkey_window;
static HANDLE hotkey_thread_handle;
static UINT hotkey_thread_id;
static HANDLE raw_input_thread_handle;
static UINT raw_input_thread_id;

/* Functions ****************************************************************/

BOOL disable_hotkeys(UINT hotkeys)
{
	if (!hotkeys)
	{
		disabled_hotkeys = 0;
		return TRUE;
	}

	disabled_hotkeys |= hotkeys;
	return TRUE;
}

/****************************************************************************/

BOOL set_hotkey_modifier(UINT modifier)
{
	if (!modifier)
	{
		hotkey_modifiers = 0;
		return TRUE;
	}

	hotkey_modifiers |= modifier;
	return TRUE;
}

/****************************************************************************/

LPCTSTR _tcsmod(LPTSTR text, LPCTSTR separator)
{
	UINT modifiers = hotkey_modifiers;
	int index;

	if (!modifiers)
		modifiers = default_modifiers;

	for (index = 0; modifier_list[index].modifier; index++)
	{
		if (modifiers & modifier_list[index].modifier)
		{
			_tcscat(text, modifier_list[index].text);
			_tcscat(text, separator);
		}
		else
		{
			if (modifiers & modifier_list[index].left_modifier)
			{
				_tcscat(text, modifier_list[index].left_text);
				_tcscat(text, separator);
			}

			if (modifiers & modifier_list[index].right_modifier)
			{
				_tcscat(text, modifier_list[index].right_text);
				_tcscat(text, separator);
			}
		}
	}

	return text;
}

/****************************************************************************/

LPCTSTR get_hotkey_text(UINT id, LPCTSTR name)
{
	int index;
	static TCHAR text[256];

	for (index = 0; hotkey_list[index].id; index++)
	{
		if (hotkey_list[index].id != id)
			continue;

		if (disabled_hotkeys & hotkey_list[index].set)
			continue;

		_tcscpy(text, name);
		_tcscat(text, _T("("));
		_tcsmod(text, _T(" "));
		_tcscat(text, hotkey_list[index].text);
		_tcscat(text, _T(")"));
		return text;
	}

	return name;
}

/****************************************************************************/

BOOL set_modifier_state(UINT modifier, BOOL pressed)
{
	if (pressed)
		modifier_state |= modifier;
	else
		modifier_state &= ~modifier;

	return TRUE;
}

/****************************************************************************/

BOOL fix_modifiers()
{
	if (!GetAsyncKeyState(VK_LMENU))
		set_modifier_state(HK_LALT, FALSE);

	if (!GetAsyncKeyState(VK_RMENU))
		set_modifier_state(HK_RALT, FALSE);

	if (!GetAsyncKeyState(VK_LCONTROL))
		set_modifier_state(HK_LCTRL, FALSE);

	if (!GetAsyncKeyState(VK_RCONTROL))
		set_modifier_state(HK_RCTRL, FALSE);

	if (!GetAsyncKeyState(VK_LSHIFT))
		set_modifier_state(HK_LSHIFT, FALSE);

	if (!GetAsyncKeyState(VK_RSHIFT))
		set_modifier_state(HK_RSHIFT, FALSE);

	if (!GetAsyncKeyState(VK_LWIN))
		set_modifier_state(HK_LWIN, FALSE);

	if (!GetAsyncKeyState(VK_RWIN))
		set_modifier_state(HK_RWIN, FALSE);

	if (!GetAsyncKeyState(VK_APPS))
		set_modifier_state(HK_MENU, FALSE);

	return TRUE;
}

/****************************************************************************/

BOOL modifiers_pressed()
{
	UINT modifiers = hotkey_modifiers;
	int index;
	BOOL left_pressed;
	BOOL right_pressed;

	if (!modifiers)
		modifiers = default_modifiers;

	for (index = 0; modifier_list[index].modifier; index++)
	{
		left_pressed = modifier_state & modifier_list[index].left_modifier;
		right_pressed = modifier_state & modifier_list[index].right_modifier;

		if (modifiers & modifier_list[index].modifier)
		{
			if (!left_pressed && !right_pressed)
				return FALSE;
		}
		else
		{
			if (modifiers & modifier_list[index].left_modifier)
			{
				if (!left_pressed)
					return FALSE;
			}
			else
			{
				if (left_pressed)
					return FALSE;
			}

			if (modifiers & modifier_list[index].right_modifier)
			{
				if (!right_pressed)
					return FALSE;
			}
			else
			{
				if (right_pressed)
					return FALSE;
			}
		}
	}

	return TRUE;
}

/****************************************************************************/

BOOL process_hotkey(UINT vkey, BOOL extended, BOOL pressed)
{
	BOOL repeat = FALSE;
	int index;

	if (!pressed)
	{
		key_state[vkey] = FALSE;
		return FALSE;
	}

	if (key_state[vkey])
		repeat = TRUE;
	else
		key_state[vkey] = TRUE;

	fix_modifiers();

	if (!modifiers_pressed())
		return FALSE;

	for (index = 0; hotkey_list[index].id; index++)
	{
		if (disabled_hotkeys & hotkey_list[index].set)
			continue;

		if (vkey != hotkey_list[index].vkey)
			continue;

		if (!extended != !hotkey_list[index].extended)
			continue;

		if (!repeat)
			PostMessage(hotkey_window, WM_HOTKEY, hotkey_list[index].id, modifier_state);

		return TRUE;
	}

	return FALSE;
}

/****************************************************************************/

LRESULT CALLBACK process_hotkey_message(int code, WPARAM wParam, LPARAM lParam)
{
	PKBDLLHOOKSTRUCT key = (PKBDLLHOOKSTRUCT)lParam;
	BOOL extended;
	BOOL pressed;

	if (code < 0)
		return CallNextHookEx(NULL, code, wParam, lParam);

	extended = key->flags & LLKHF_EXTENDED;
	pressed = ~key->flags & LLKHF_UP;

	if (!process_hotkey(key->vkCode, extended, pressed))
		return CallNextHookEx(NULL, code, wParam, lParam);

	return TRUE;
}

/****************************************************************************/

UINT WINAPI hotkey_thread(LPVOID parameter)
{
	HHOOK hook;
	MSG message;

	hook = SetWindowsHookEx(WH_KEYBOARD_LL, process_hotkey_message, NULL, 0);

	if (!hook)
		return 1;

	while (GetMessage(&message, NULL, 0, 0) > 0)
	{
		if (message.message == WM_CLOSE)
			PostQuitMessage(0);

		TranslateMessage(&message);
		DispatchMessage(&message);
	}

	UnhookWindowsHookEx(hook);
	return 0;
}

/****************************************************************************/

BOOL process_raw_input(HRAWINPUT keyboard)
{
	RAWINPUT input;
	UINT size = sizeof input;
	BOOL extended;
	BOOL pressed;

	if (GetRawInputData(keyboard, RID_INPUT, &input, &size, sizeof input.header) <= 0)
		return FALSE;

	extended = input.data.keyboard.Flags & RI_KEY_E0;
	pressed = ~input.data.keyboard.Flags & RI_KEY_BREAK;

	switch (input.data.keyboard.MakeCode)
	{
		case 0x38:
			if (!extended)
				set_modifier_state(HK_LALT, pressed);
			else
				set_modifier_state(HK_RALT, pressed);

			break;

		case 0x1D:
			if (!extended)
				set_modifier_state(HK_LCTRL, pressed);
			else
				set_modifier_state(HK_RCTRL, pressed);

			break;

		case 0x2A:
			set_modifier_state(HK_LSHIFT, pressed);
			break;

		case 0x36:
			set_modifier_state(HK_RSHIFT, pressed);
			break;

		case 0x5B:
			if (extended)
				set_modifier_state(HK_LWIN, pressed);

			break;

		case 0x5C:
			if (extended)
				set_modifier_state(HK_RWIN, pressed);

			break;

		case 0x5D:
			if (extended)
				set_modifier_state(HK_MENU, pressed);

			break;
	}
/*
	if (!process_hotkey(input.data.keyboard.VKey, extended, pressed))
		return FALSE;
*/
	return TRUE;
}

/****************************************************************************/

BOOL CALLBACK process_raw_input_message(HWND window, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INPUT:
			process_raw_input((HRAWINPUT)lParam);
			return TRUE;
	}

	return FALSE;
}

/****************************************************************************/

UINT WINAPI raw_input_thread(LPVOID parameter)
{
	HWND window;
	RAWINPUTDEVICE keyboard;
	MSG message;

	window = CreateDialog(GetModuleHandle(NULL), _T("IDD_MESSAGE"), NULL, process_raw_input_message);

	if (!window)
		return 1;

	keyboard.usUsagePage = 1;
	keyboard.usUsage = 6;
	keyboard.dwFlags = RIDEV_INPUTSINK;
	keyboard.hwndTarget = window;

	if (!RegisterRawInputDevices(&keyboard, 1, sizeof keyboard))
	{
		DestroyWindow(window);
		return 1;
	}

	while (GetMessage(&message, NULL, 0, 0) > 0)
	{
		if (message.message == WM_CLOSE)
			PostQuitMessage(0);

		TranslateMessage(&message);
		DispatchMessage(&message);
	}

	DestroyWindow(window);
	return 0;
}

/****************************************************************************/

BOOL register_hotkeys(HWND window)
{
	if (disabled_hotkeys == HKS_ALL)
		return FALSE;

	if (!window)
		return FALSE;

	hotkey_window = window;
	modifier_state = 0;
	memset(key_state, 0, sizeof key_state);

	// Create a separate thread for the keyboard hook to avoid timeouts
	hotkey_thread_handle = (HANDLE)_beginthreadex(NULL, 0, hotkey_thread, NULL, 0, &hotkey_thread_id);

	if (!hotkey_thread_handle)
	{
		hotkey_thread_id = 0;
		return FALSE;
	}

	// Create a separate thread for processing raw input
	raw_input_thread_handle = (HANDLE)_beginthreadex(NULL, 0, raw_input_thread, NULL, 0, &raw_input_thread_id);

	if (!raw_input_thread_handle)
	{
		raw_input_thread_id = 0;
		return FALSE;
	}

	return TRUE;
}

/****************************************************************************/

BOOL flush_hotkeys()
{
	MSG message;
	BOOL message_exists = FALSE;

	// Remove all the pending WM_HOTKEY messages
	while (PeekMessage(&message, NULL, WM_HOTKEY, WM_HOTKEY, PM_REMOVE))
		message_exists = TRUE;

	// Process the last WM_HOTKEY message if available
	if (message_exists)
	{
		TranslateMessage(&message);
		DispatchMessage(&message);
	}

	return TRUE;
}

/****************************************************************************/

BOOL unregister_hotkeys()
{
	if (raw_input_thread_id)
	{
		PostThreadMessage(raw_input_thread_id, WM_CLOSE, 0, 0);
		raw_input_thread_id = 0;
	}

	if (raw_input_thread_handle)
	{
		WaitForSingleObject(raw_input_thread_handle, 2000);
		CloseHandle(raw_input_thread_handle);
		raw_input_thread_handle = 0;
	}

	if (hotkey_thread_id)
	{
		PostThreadMessage(hotkey_thread_id, WM_CLOSE, 0, 0);
		hotkey_thread_id = 0;
	}

	if (hotkey_thread_handle)
	{
		WaitForSingleObject(hotkey_thread_handle, 2000);
		CloseHandle(hotkey_thread_handle);
		hotkey_thread_handle = 0;
	}

	return TRUE;
}

/****************************************************************************/
