/* Includes *****************************************************************/

#include "i2c.h"
#include "api_amd.h"
#include "api_nvidia.h"
#include "debug.h"
#include "displays.h"
#include "i2c_amd.h"
#include "i2c_nvidia.h"

/* Constants ****************************************************************/

const int I2C_READ = 0;
const int I2C_WRITE = 1;

/* Functions ****************************************************************/

BOOL i2c_send(struct display *display, int delay, unsigned char address, unsigned char *data, int size)
{
	if (display->adapter->vendor == ADAPTER_AMD)
		return i2c_amd(I2C_WRITE, display, delay, address, data, size);

	if (display->adapter->vendor == ADAPTER_NVIDIA)
		return i2c_nvidia(I2C_WRITE, display, delay, address, data, size);

	return FALSE;
}

/****************************************************************************/

BOOL i2c_recv(struct display *display, int delay, unsigned char address, unsigned char *data, int size)
{
	if (display->adapter->vendor == ADAPTER_AMD)
		return i2c_amd(I2C_READ, display, delay, address, data, size);

	if (display->adapter->vendor == ADAPTER_NVIDIA)
		return i2c_nvidia(I2C_READ, display, delay, address, data, size);

	return FALSE;
}

/****************************************************************************/

BOOL i2c_debug(int action, struct display *display, int delay, unsigned char address, unsigned char *data, int size, int status)
{
	int i;

	if (action >= get_debug_level())
		return TRUE;

	if (!debug_file())
		return FALSE;

	debug_timestamp();

	if (action == I2C_READ)
		fprintf(debug_file(), "RECV");
	else if (action == I2C_WRITE)
		fprintf(debug_file(), "SEND");

	fprintf(debug_file(), "(%s, %i, %i, %i.%03i): <%02X>", display->adapter->vendor, display->adapter->id, display->id, delay / 1000, delay % 1000, address);

	for (i = action; i < size; i++)
		fprintf(debug_file(), " %02X", data[i]);

	if (status)
		fprintf(debug_file(), " = ERROR %i", status);

	fprintf(debug_file(), "\n");
	fflush(debug_file());
	return TRUE;
}

/****************************************************************************/
