/* Includes *****************************************************************/

#include "icon.h"

/* Structs ******************************************************************/

static struct
{
	const UINT id;
	const LPCTSTR tip;
	HICON handle;
}
notify_icon_list[] =
{
	{IDI_STROBING_OFF,	STROBELIGHT_VERSION},
	{IDI_STROBING_ON,	STROBELIGHT_VERSION},
	{0}
};

/* Locals *******************************************************************/

static BOOL notify_icon_enabled = TRUE;
static PNOTIFYICONDATA notify_icon;
static HANDLE notify_icon_thread_handle;
static UINT notify_icon_thread_id;

/* Functions ****************************************************************/

BOOL enable_notify_icon(BOOL enabled)
{
	notify_icon_enabled = enabled;
	return TRUE;
}

/****************************************************************************/

BOOL CALLBACK process_notify_icon_message(HWND window, UINT message, WPARAM wParam, LPARAM lParam)
{
	static BOOL action;

	switch (message)
	{
		case WM_INITDIALOG:
			if (!Shell_NotifyIcon(NIM_ADD, notify_icon))
			{
				action = NIM_ADD;
				SetTimer(window, 1, 200, NULL);
				return FALSE;
			}

			action = NIM_MODIFY;
			return FALSE;		// Return FALSE to avoid activating the window so hotkeys work properly

		case WM_TIMER:
			if (!Shell_NotifyIcon(action, notify_icon))
				return FALSE;

			KillTimer(window, 1);
			action = NIM_MODIFY;
			return TRUE;

		case WM_APP:
			notify_icon->hIcon = notify_icon_list[wParam].handle;
			_tcscpy(notify_icon->szTip, notify_icon_list[wParam].tip);

			if (!Shell_NotifyIcon(action, notify_icon))
			{
				SetTimer(window, 1, 100, NULL);
				return FALSE;
			}

			action = NIM_MODIFY;
			return TRUE;
	}

	return FALSE;
}

/****************************************************************************/

UINT WINAPI notify_icon_thread(LPVOID parameter)
{
	HWND window;
	MSG message;

	window = CreateDialog(GetModuleHandle(NULL), _T("IDD_MESSAGE"), NULL, process_notify_icon_message);

	if (!window)
		return 1;

	while (GetMessage(&message, NULL, 0, 0) > 0)
	{
		if (message.message == WM_CLOSE)
			PostQuitMessage(0);

		if (message.message == WM_APP)
			message.hwnd = window;

		TranslateMessage(&message);
		DispatchMessage(&message);
	}

	DestroyWindow(window);
	return 0;
}

/****************************************************************************/

BOOL add_notify_icon(HWND window, int id)
{
	int index;

	if (!notify_icon_enabled)
		return TRUE;

	if (!window)
		return FALSE;

	delete_notify_icon();

	for (index = 0; notify_icon_list[index].id; index++)
		notify_icon_list[index].handle = (HICON)LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(notify_icon_list[index].id), IMAGE_ICON, 16, 16, 0);

	notify_icon = (PNOTIFYICONDATA)malloc(sizeof *notify_icon);
	memset(notify_icon, 0, sizeof *notify_icon);
	notify_icon->cbSize = sizeof *notify_icon;
	notify_icon->hWnd = window;
	notify_icon->uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
	notify_icon->uCallbackMessage = WM_APP;
	notify_icon->hIcon = notify_icon_list[id].handle;
	_tcscpy(notify_icon->szTip, notify_icon_list[id].tip);

	// Create a separate thread for updating the notify icon
	notify_icon_thread_handle = (HANDLE)_beginthreadex(NULL, 0, notify_icon_thread, NULL, 0, &notify_icon_thread_id);

	if (!notify_icon_thread_handle)
	{
		notify_icon_thread_id = 0;
		return FALSE;
	}

	return TRUE;
}

/****************************************************************************/

BOOL update_notify_icon(int id)
{
	if (!notify_icon_thread_id)
		return FALSE;

	PostThreadMessage(notify_icon_thread_id, WM_APP, id, 0);
	return TRUE;
}

/****************************************************************************/

BOOL delete_notify_icon()
{
	int index;

	if (notify_icon_thread_id)
	{
		PostThreadMessage(notify_icon_thread_id, WM_CLOSE, 0, 0);
		notify_icon_thread_id = 0;
	}

	if (notify_icon_thread_handle)
	{
		WaitForSingleObject(notify_icon_thread_handle, 2000);
		CloseHandle(notify_icon_thread_handle);
		notify_icon_thread_handle = 0;
	}

	if (notify_icon)
	{
		Shell_NotifyIcon(NIM_DELETE, notify_icon);
		free(notify_icon);
		notify_icon = NULL;
	}

	for (index = 0; notify_icon_list[index].id; index++)
	{
		if (notify_icon_list[index].handle)
		{
			DestroyIcon(notify_icon_list[index].handle);
			notify_icon_list[index].handle = 0;
		}
	}

	return TRUE;
}

/****************************************************************************/
