/* Includes *****************************************************************/

#include "init.h"
#include "debug.h"
#include "dialog.h"
#include "displays.h"
#include "i2c.h"

/* Constants ****************************************************************/

static const int MAX_RETRIES = 10;
static const int DELAY_DEFAULT = 200;
static const int DELAY_MINIMUM = 40;
static const int DELAY_INCREMENT = 10;
static const int DELAY_MAXIMUM = 500;

/* Functions ****************************************************************/

BOOL init_display(struct display *display, HWND window)
{
	int delay;
	int passes;
	unsigned char reply[256] = {0};
	unsigned char command_get_3d_status[] = {0x6E, 0x51, 0x81, 0x59, 0xE7};
	unsigned char command_get_3d_status_size = sizeof command_get_3d_status;
	unsigned char reply_3d_enabled[] = {0x6E, 0x82, 0x58, 0x01, 0xE5};
	unsigned char reply_3d_disabled[] = {0x6E, 0x82, 0x58, 0x00, 0xE4};
	unsigned char reply_3d_status_size = sizeof reply_3d_enabled;
	unsigned char command_get_vendor[] = {0x6E, 0x51, 0x81, 0x5A, 0xE4};
	unsigned char command_get_vendor_size = sizeof command_get_vendor;
	unsigned char reply_vendor_acer[] = {0x6E, 0x8B, 0x5B, 0x4D, 0x53, 0x54, 0x41, 0x52, 0x20, 0x33, 0x44, 0x00, 0x02, 0xE2};
	unsigned char reply_vendor_asus[] = {0x6E, 0x8B, 0x5B, 0x4D, 0x53, 0x54, 0x41, 0x52, 0x20, 0x33, 0x44, 0x00, 0x06, 0xE6};
	unsigned char reply_vendor_benq[] = {0x6E, 0x8B, 0x5B, 0x4D, 0x53, 0x54, 0x41, 0x52, 0x20, 0x33, 0x44, 0x00, 0x10, 0xF0};
	unsigned char reply_vendor_size = sizeof reply_vendor_asus;
	unsigned char *command_handshake;
	unsigned char command_handshake_asus[] = {0x6E, 0x51, 0x99, 0x50, 0x43, 0x6F, 0x70, 0x79, 0x72, 0x69, 0x67, 0x68, 0x74, 0x20, 0x4E, 0x56, 0x49, 0x44, 0x49, 0x41, 0x90, 0xC4, 0x09, 0x9C, 0x07, 0x5E, 0xFD, 0xB8, 0x53};
	unsigned char command_handshake_benq[] = {0x6E, 0x51, 0x99, 0x50, 0x43, 0x6F, 0x70, 0x79, 0x72, 0x69, 0x67, 0x68, 0x74, 0x20, 0x4E, 0x56, 0x49, 0x44, 0x49, 0x41, 0xAA, 0xBD, 0xB8, 0xDA, 0x2D, 0x67, 0xCA, 0x1F, 0x64};
	unsigned char command_handshake_size = sizeof command_handshake_asus;
	unsigned char command_get_handshake_reply[] = {0x6E, 0x51, 0x81, 0x55, 0xEB};
	unsigned char command_get_handshake_reply_size = sizeof command_get_handshake_reply;
	unsigned char *reply_handshake;
	unsigned char reply_handshake_asus[] = {0x6E, 0x85, 0x54, 0x96, 0x04, 0xCA, 0xC3, 0x74};
	unsigned char reply_handshake_benq[] = {0x6E, 0x85, 0x54, 0x22, 0x03, 0x0E, 0xE9, 0x29};
	unsigned char reply_handshake_size = sizeof reply_handshake_asus;
	unsigned char *command_enable_3d;
	unsigned char command_enable_3d_asus[] = {0x6E, 0x51, 0x95, 0x56, 0xF4, 0x05, 0x32, 0xEA, 0x53, 0x1F, 0x41, 0x2A, 0xDD, 0xBA, 0x29, 0x19, 0xD0, 0x2D, 0x4C, 0xFF, 0x94, 0x36, 0xDA, 0x29, 0xBA};
	unsigned char command_enable_3d_benq[] = {0x6E, 0x51, 0x95, 0x56, 0x2B, 0xB9, 0x79, 0x60, 0x29, 0x1D, 0x8A, 0x5F, 0xAB, 0xBD, 0xD3, 0x16, 0xFE, 0x96, 0x94, 0x20, 0xAE, 0xD3, 0x82, 0x33, 0x55};
	unsigned char command_enable_3d_size = sizeof command_enable_3d_asus;

	for (delay = DELAY_DEFAULT, passes = 1;; passes++)
	{
		if (!i2c_send(display, 0, 0x6E, command_get_3d_status, command_get_3d_status_size))
			return FALSE;

		if (!i2c_recv(display, delay, 0x6F, reply, reply_3d_status_size))
			return FALSE;

		if (memcmp(reply, reply_3d_enabled, reply_3d_status_size) == 0)
		{
			debug_printf("INIT(%s, %i, %i): 3D = TRUE", display->adapter->vendor, display->adapter->id, display->id);
			return TRUE;
		}

		if (memcmp(reply, reply_3d_disabled, reply_3d_status_size) == 0)
		{
			debug_printf("INIT(%s, %i, %i): 3D = FALSE", display->adapter->vendor, display->adapter->id, display->id);
			break;
		}

		if (passes == MAX_RETRIES)
		{
			debug_printf("INIT(%s, %i, %i): 3D = ERROR", display->adapter->vendor, display->adapter->id, display->id);
			return FALSE;
		}
	}

	for (delay = DELAY_DEFAULT, passes = 1;; passes++)
	{
		if (!i2c_send(display, 0, 0x6E, command_get_vendor, command_get_vendor_size))
			return FALSE;

		if (!i2c_recv(display, delay, 0x6F, reply, reply_vendor_size))
			return FALSE;

		if (memcmp(reply, reply_vendor_acer, reply_vendor_size) == 0)
		{
			debug_printf("INIT(%s, %i, %i): VENDOR = 0x%02X (Acer)", display->adapter->vendor, display->adapter->id, display->id, reply[12]);
			return TRUE;
		}

		if (memcmp(reply, reply_vendor_asus, reply_vendor_size) == 0)
		{
			debug_printf("INIT(%s, %i, %i): VENDOR = 0x%02X (ASUS)", display->adapter->vendor, display->adapter->id, display->id, reply[12]);
			command_handshake = command_handshake_asus;
			reply_handshake = reply_handshake_asus;
			command_enable_3d = command_enable_3d_asus;
			break;
		}

		if (memcmp(reply, reply_vendor_benq, reply_vendor_size) == 0)
		{
			debug_printf("INIT(%s, %i, %i): VENDOR = 0x%02X (BenQ)", display->adapter->vendor, display->adapter->id, display->id, reply[12]);
			command_handshake = command_handshake_benq;
			reply_handshake = reply_handshake_benq;
			command_enable_3d = command_enable_3d_benq;
			break;
		}

		if (passes == MAX_RETRIES)
		{
			debug_printf("INIT(%s, %i, %i): VENDOR = 0x%02X (unsupported)", display->adapter->vendor, display->adapter->id, display->id, reply[12]);
			return FALSE;
		}
	}

	debug_printf("INIT(%s, %i, %i): Initializing display %i...", display->adapter->vendor, display->adapter->id, display->id, display->number);
	ShowWindow(window, SW_SHOW);

	for (delay = DELAY_MINIMUM, passes = 1;; passes++)
	{
		dialog_printf(window, IDC_STATUS, _T(" Initializing display %i... (Pass: %i)"), display->number, passes);

		if (!i2c_send(display, 0, 0x6E, command_handshake, command_handshake_size))
			return FALSE;

		if (!i2c_send(display, delay, 0x6E, command_get_handshake_reply, command_get_handshake_reply_size))
			return FALSE;

		if (!i2c_recv(display, delay, 0x6F, reply, reply_handshake_size))
			return FALSE;

		if (memcmp(reply, reply_handshake, reply_handshake_size) == 0)
			break;

		if (memcmp(reply, reply_handshake, 3) == 0)
			continue;

		delay += DELAY_INCREMENT;

		if (delay > DELAY_MAXIMUM)
			return FALSE;

		Sleep(DELAY_DEFAULT);
	}

	debug_printf("INIT(%s, %i, %i): DELAY = %i", display->adapter->vendor, display->adapter->id, display->id, delay);
	debug_printf("INIT(%s, %i, %i): PASSES = %i", display->adapter->vendor, display->adapter->id, display->id, passes);

	if (!i2c_send(display, DELAY_DEFAULT, 0x6E, command_enable_3d, command_enable_3d_size))
		return FALSE;

	if (!i2c_send(display, DELAY_DEFAULT, 0x6E, command_get_3d_status, command_get_3d_status_size))
		return FALSE;

	if (!i2c_recv(display, DELAY_DEFAULT, 0x6F, reply, reply_3d_status_size))
		return FALSE;

	if (memcmp(reply, reply_3d_enabled, reply_3d_status_size) == 0)
	{
		debug_printf("INIT(%s, %i, %i): 3D = TRUE", display->adapter->vendor, display->adapter->id, display->id);
		display->restart = TRUE;
		return TRUE;
	}

	if (memcmp(reply, reply_3d_disabled, reply_3d_status_size) == 0)
	{
		debug_printf("INIT(%s, %i, %i): 3D = FALSE", display->adapter->vendor, display->adapter->id, display->id);
		return FALSE;
	}

	debug_printf("INIT(%s, %i, %i): 3D = ERROR", display->adapter->vendor, display->adapter->id, display->id);
	return FALSE;
}

/****************************************************************************/
