/* Includes *****************************************************************/

#include "init_amd.h"
#include "api_amd.h"
#include "debug.h"
#include "displays.h"
#include "init.h"

/* Functions ****************************************************************/

int init_amd(struct display_list *display_list, HWND window)
{
	int status;
	int adapter_count;
	AdapterInfo adapter_info[ADL_MAX_ADAPTERS];
	int adapter_index;
	int display_count;
	ADLDisplayInfo *display_info;
	int display_index;
	struct display *display;
	int result = 0;

	if (!display_list)
		return FALSE;

	status = ADL_Adapter_NumberOfAdapters_Get(&adapter_count);

	if (status != ADL_OK)
	{
		debug_printf("INIT(%s): ADL_Adapter_NumberOfAdapters_Get() = ERROR %i", ADAPTER_AMD, status);
		return FALSE;
	}

	status = ADL_Adapter_AdapterInfo_Get(adapter_info, sizeof adapter_info);

	if (status != ADL_OK)
	{
		debug_printf("INIT(%s): ADL_Adapter_AdapterInfo_Get() = ERROR %i", ADAPTER_AMD, status);
		return FALSE;
	}

	for (adapter_index = 0; adapter_index < adapter_count; adapter_index++)
	{
		display_info = NULL;
		status = ADL_Display_DisplayInfo_Get(adapter_info[adapter_index].iAdapterIndex, &display_count, &display_info, 0);

		if (status != ADL_OK)
		{
			debug_printf("INIT(%s, %i): ADL_Display_DisplayInfo_Get() = ERROR %i", ADAPTER_AMD, adapter_index, status);
			continue;
		}

		for (display_index = 0; display_index < display_count; display_index++)
		{
			if (display_info[display_index].displayID.iDisplayLogicalAdapterIndex == adapter_info[adapter_index].iAdapterIndex)
			{
				display = new_display();
				display->number = display_list->count + 1;
				display->id = display_index;
				display->amd = display_info[display_index].displayID.iDisplayLogicalIndex;
				display->adapter = new_adapter();
				display->adapter->vendor = ADAPTER_AMD;
				display->adapter->id = adapter_index;
				display->adapter->amd = adapter_info[adapter_index].iAdapterIndex;

				if (!init_display(display, window))
				{
					free(display->adapter);
					free(display);
					continue;
				}

				add_display(display_list, display);
				result++;
			}
		}

		free(display_info);
	}

	return result;
}

/****************************************************************************/
