/* Includes *****************************************************************/

#include "init_nvidia.h"
#include "api_nvidia.h"
#include "debug.h"
#include "displays.h"
#include "init.h"

/* Functions ****************************************************************/

int init_nvidia(struct display_list *display_list, HWND window)
{
	int status;
	NvPhysicalGpuHandle adapter_info[NVAPI_MAX_PHYSICAL_GPUS];
	NvU32 adapter_count;
	NvU32 adapter_index;
	NvU32 output_mask;
	NvU32 output_index;
	NvU32 output_id;
	struct display *display;
	int result = 0;

	if (!display_list)
		return FALSE;

	status = NvAPI_EnumPhysicalGPUs(adapter_info, &adapter_count);

	if (status != NVAPI_OK)
	{
		debug_printf("INIT(%s): NvAPI_EnumPhysicalGPUs() = ERROR %i", ADAPTER_NVIDIA, status);
		return FALSE;
	}

	for (adapter_index = 0; adapter_index < adapter_count; adapter_index++)
	{
		status = NvAPI_GPU_GetActiveOutputs(adapter_info[adapter_index], &output_mask);

		if (status != NVAPI_OK)
		{
			debug_printf("INIT(%s, %i): NvAPI_GPU_GetActiveOutputs() = ERROR %i", ADAPTER_NVIDIA, adapter_index, status);
			continue;
		}

		for (output_index = 0; output_index < NVAPI_MAX_HEADS_PER_GPU; output_index++)
		{
			output_id = 1 << output_index;

			if (output_mask & output_id)
			{
				display = new_display();
				display->number = display_list->count + 1;
				display->id = output_index;
				display->nvidia = output_id;
				display->adapter = new_adapter();
				display->adapter->vendor = ADAPTER_NVIDIA;
				display->adapter->id = adapter_index;
				display->adapter->nvidia = adapter_info[adapter_index];

				if (!init_display(display, window))
				{
					free(display->adapter);
					free(display);
					continue;
				}

				add_display(display_list, display);
				result++;
			}
		}
	}

	return result;
}

/****************************************************************************/
