/* Includes *****************************************************************/

#include "menu.h"
#include "hotkeys.h"
#include "strobing.h"

/* Locals *******************************************************************/

static HMENU menu;
static HMENU brightness_menu;

/* Functions ****************************************************************/

BOOL load_notify_menu()
{
	if (menu)
		DestroyMenu(menu);

	menu = CreatePopupMenu();

	if (!menu)
		return FALSE;

	AppendMenu(menu, MF_STRING, IDM_REINITIALIZE, _T("Reinitialize displays"));
	AppendMenu(menu, MF_SEPARATOR, 0, 0);
	AppendMenu(menu, !is_strobing_on_available(), IDM_STROBING_ON, get_hotkey_text(IDM_STROBING_ON, _T("Strobing on    ")));
	AppendMenu(menu, !is_strobing_off_available(), IDM_STROBING_OFF, get_hotkey_text(IDM_STROBING_OFF, _T("Strobing off    ")));
	brightness_menu = CreatePopupMenu();

	if (brightness_menu)
	{
		AppendMenu(menu, MF_SEPARATOR, 0, 0);
		AppendMenu(brightness_menu, MF_STRING, IDM_BRIGHTNESS_10, get_hotkey_text(IDM_BRIGHTNESS_10, _T("10%\t")));
		AppendMenu(brightness_menu, MF_STRING, IDM_BRIGHTNESS_20, get_hotkey_text(IDM_BRIGHTNESS_20, _T("20%\t")));
		AppendMenu(brightness_menu, MF_STRING, IDM_BRIGHTNESS_30, get_hotkey_text(IDM_BRIGHTNESS_30, _T("30%\t")));
		AppendMenu(brightness_menu, MF_STRING, IDM_BRIGHTNESS_40, get_hotkey_text(IDM_BRIGHTNESS_40, _T("40%\t")));
		AppendMenu(brightness_menu, MF_STRING, IDM_BRIGHTNESS_50, get_hotkey_text(IDM_BRIGHTNESS_50, _T("50%\t")));
		AppendMenu(brightness_menu, MF_STRING, IDM_BRIGHTNESS_60, get_hotkey_text(IDM_BRIGHTNESS_60, _T("60%\t")));
		AppendMenu(brightness_menu, MF_STRING, IDM_BRIGHTNESS_70, get_hotkey_text(IDM_BRIGHTNESS_70, _T("70%\t")));
		AppendMenu(brightness_menu, MF_STRING, IDM_BRIGHTNESS_80, get_hotkey_text(IDM_BRIGHTNESS_80, _T("80%\t")));
		AppendMenu(brightness_menu, MF_STRING, IDM_BRIGHTNESS_90, get_hotkey_text(IDM_BRIGHTNESS_90, _T("90%\t")));
		AppendMenu(brightness_menu, MF_STRING, IDM_BRIGHTNESS_100, get_hotkey_text(IDM_BRIGHTNESS_100, _T("100%\t")));
		AppendMenu(menu, MF_POPUP, (UINT_PTR)brightness_menu, _T("Brightness"));
	}

	AppendMenu(menu, MF_SEPARATOR, 0, 0);
	AppendMenu(menu, MF_STRING, IDM_STROBELIGHT_LINK, _T("Strobelight Home Page"));
	AppendMenu(menu, MF_STRING, IDM_BLURBUSTERS_LINK, _T("Blur Busters Blog"));
	AppendMenu(menu, MF_STRING, IDM_TESTUFO_LINK, _T("TestUFO Motion Tests"));
	AppendMenu(menu, MF_SEPARATOR, 0, 0);
	AppendMenu(menu, MF_STRING, IDM_EXIT, _T("Exit"));
	return TRUE;
}

/****************************************************************************/

BOOL unload_notify_menu()
{
	if (menu)
	{
		DestroyMenu(menu);
		menu = NULL;
	}

	return TRUE;
}

/****************************************************************************/

BOOL show_notify_menu(HWND window, BOOL strobed)
{
	POINT point;

	if (!window)
		return FALSE;

	if (!menu)
		return FALSE;

	if (strobed)
	{
		CheckMenuItem(menu, IDM_STROBING_ON, MF_CHECKED);
		CheckMenuItem(menu, IDM_STROBING_OFF, MF_UNCHECKED);

		if (brightness_menu)
			ModifyMenu(menu, (UINT)brightness_menu, MF_POPUP, (UINT)brightness_menu, _T("Strobe brightness"));
	}
	else
	{
		CheckMenuItem(menu, IDM_STROBING_ON, MF_UNCHECKED);
		CheckMenuItem(menu, IDM_STROBING_OFF, MF_CHECKED);

		if (brightness_menu)
			ModifyMenu(menu, (UINT)brightness_menu, MF_POPUP, (UINT)brightness_menu, _T("Brightness"));
	}

	GetCursorPos(&point);
	SetForegroundWindow(window);	// Activate the window so the menu works properly
	TrackPopupMenu(menu, 0, point.x, point.y, 0, window, NULL);
	SetActiveWindow(NULL);			// Deactivate the window so hotkeys work properly
	return TRUE;
}

/****************************************************************************/
