/* Includes *****************************************************************/

#include "strobing.h"

/* Constants ****************************************************************/

#ifndef SWAPPED
static const int strobing_on[] = {120, 110, 100, 0};
static const int strobing_off[] = {144, 121, 111, 101, 0};
#else
static const int strobing_on[] = {121, 111, 101, 0};
static const int strobing_off[] = {144, 120, 110, 100, 0};
#endif

/* Structs ******************************************************************/

struct strobing_display
{
	DISPLAY_DEVICE device;
	int strobing_on;
	int strobing_off;
	struct strobing_display *next;
};

/****************************************************************************/

struct strobing_display_list
{
	int count;
	struct strobing_display *first;
	struct strobing_display *last;
};

/* Locals *******************************************************************/

static struct strobing_display_list *strobing_displays;

/* Functions ****************************************************************/

struct strobing_display *new_strobing_display()
{
	struct strobing_display *strobing_display;

	strobing_display = (struct strobing_display *)malloc(sizeof *strobing_display);
	memset(strobing_display, 0, sizeof *strobing_display);
	return strobing_display;
}

/****************************************************************************/

struct strobing_display_list *new_strobing_display_list()
{
	struct strobing_display_list *strobing_displays;

	strobing_displays = (struct strobing_display_list *)malloc(sizeof *strobing_displays);
	memset(strobing_displays, 0, sizeof *strobing_displays);
	return strobing_displays;
}

/****************************************************************************/

BOOL add_strobing_display(struct strobing_display_list *strobing_displays, struct strobing_display *strobing_display)
{
	if (!strobing_displays || !strobing_display)
		return FALSE;

	if (!strobing_displays->first)
		strobing_displays->first = strobing_display;
	else
		strobing_displays->last->next = strobing_display;

	strobing_displays->last = strobing_display;
	strobing_displays->count++;
	return TRUE;
}

/****************************************************************************/

void delete_strobing_display_list(struct strobing_display_list *strobing_displays)
{
	struct strobing_display *strobing_display;
	struct strobing_display *next;

	for (strobing_display = strobing_displays->first; strobing_display; strobing_display = next)
	{
		next = strobing_display->next;
		free(strobing_display);
	}

	free(strobing_displays);
}

/****************************************************************************/

int test_device_frequencies(DISPLAY_DEVICE device, const int *frequencies)
{
	DEVMODE mode;
	int index;

	if (!frequencies)
		return FALSE;

	mode.dmSize = sizeof mode;

	if (!EnumDisplaySettings(device.DeviceName, ENUM_CURRENT_SETTINGS, &mode))
		return FALSE;

	mode.dmDisplayFlags = 0;

	for (index = 0; frequencies[index]; index++)
	{
		mode.dmDisplayFrequency = frequencies[index];

		if (ChangeDisplaySettingsEx(device.DeviceName, &mode, NULL, CDS_TEST, NULL) == DISP_CHANGE_SUCCESSFUL)
			return frequencies[index];
	}

	return FALSE;
}

/****************************************************************************/

int load_strobing_displays()
{
	int index;
	DISPLAY_DEVICE device;
	struct strobing_display *strobing_display;
	int result = 0;

	if (strobing_displays)
		delete_strobing_display_list(strobing_displays);

	strobing_displays = new_strobing_display_list();

	for (index = 0; device.cb = sizeof device, EnumDisplayDevices(NULL, index, &device, 0); index++)
	{
		strobing_display = new_strobing_display();
		strobing_display->device = device;
		strobing_display->strobing_on = test_device_frequencies(device, strobing_on);
		strobing_display->strobing_off = test_device_frequencies(device, strobing_off);
		add_strobing_display(strobing_displays, strobing_display);

		if (strobing_display->strobing_on)
			result++;
	}

	return result;
}

/****************************************************************************/

BOOL unload_strobing_displays()
{
	delete_strobing_display_list(strobing_displays);
	strobing_displays = NULL;
	return TRUE;
}

/****************************************************************************/

BOOL is_any_display_switchable()
{
	struct strobing_display *strobing_display;

	for (strobing_display = strobing_displays->first; strobing_display; strobing_display = strobing_display->next)
		if (strobing_display->strobing_on && strobing_display->strobing_off)
			return TRUE;

	return FALSE;
}

/****************************************************************************/

BOOL is_strobing_on_available()
{
	struct strobing_display *strobing_display;

	for (strobing_display = strobing_displays->first; strobing_display; strobing_display = strobing_display->next)
		if (strobing_display->strobing_on)
			return TRUE;

	return FALSE;
}

/****************************************************************************/

BOOL is_strobing_off_available()
{
	if (is_any_display_switchable())
		return TRUE;

	if (is_strobing_on_available())
		return FALSE;

	return TRUE;
}

/****************************************************************************/

BOOL is_device_strobed(DISPLAY_DEVICE device)
{
	DEVMODE mode;
	int index;

	mode.dmSize = sizeof mode;

	if (!EnumDisplaySettings(device.DeviceName, ENUM_CURRENT_SETTINGS, &mode))
		return FALSE;

	for (index = 0; strobing_on[index]; index++)
		if (mode.dmDisplayFrequency == strobing_on[index])
			return TRUE;

	return FALSE;
}

/****************************************************************************/

BOOL is_any_strobing_display_not_strobed()
{
	struct strobing_display *strobing_display;

	for (strobing_display = strobing_displays->first; strobing_display; strobing_display = strobing_display->next)
		if (strobing_display->strobing_on)
			if (!is_device_strobed(strobing_display->device))
				return TRUE;

	return FALSE;
}

/****************************************************************************/

BOOL is_any_switchable_display_strobed()
{
	struct strobing_display *strobing_display;

	for (strobing_display = strobing_displays->first; strobing_display; strobing_display = strobing_display->next)
		if (strobing_display->strobing_on && strobing_display->strobing_off)
			if (is_device_strobed(strobing_display->device))
				return TRUE;

	return FALSE;
}

/****************************************************************************/

BOOL is_any_switchable_display_not_strobed()
{
	struct strobing_display *strobing_display;

	for (strobing_display = strobing_displays->first; strobing_display; strobing_display = strobing_display->next)
		if (strobing_display->strobing_on && strobing_display->strobing_off)
			if (!is_device_strobed(strobing_display->device))
				return TRUE;

	return FALSE;
}

/****************************************************************************/

BOOL is_any_display_strobed()
{
	struct strobing_display *strobing_display;

	for (strobing_display = strobing_displays->first; strobing_display; strobing_display = strobing_display->next)
		if (is_device_strobed(strobing_display->device))
			return TRUE;

	return FALSE;
}

/****************************************************************************/

BOOL get_strobed()
{
	if (is_any_strobing_display_not_strobed())
		return FALSE;

	if (is_any_display_strobed())
		return TRUE;

	return FALSE;
}

/****************************************************************************/

BOOL set_device_frequency(DISPLAY_DEVICE device, int frequency)
{
	DEVMODE mode;

	if (!frequency)
		return FALSE;

	mode.dmSize = sizeof mode;

	if (!EnumDisplaySettings(device.DeviceName, ENUM_CURRENT_SETTINGS, &mode))
		return FALSE;

	mode.dmDisplayFlags = 0;
	mode.dmDisplayFrequency = frequency;

	if (ChangeDisplaySettingsEx(device.DeviceName, &mode, NULL, CDS_UPDATEREGISTRY, NULL) == DISP_CHANGE_SUCCESSFUL)
		return TRUE;

	return FALSE;
}

/****************************************************************************/

int set_strobed(BOOL strobed)
{
	struct strobing_display *strobing_display;
	int result = 0;

	if (strobed)
	{
		for (strobing_display = strobing_displays->first; strobing_display; strobing_display = strobing_display->next)
			if (strobing_display->strobing_on)
				if (set_device_frequency(strobing_display->device, strobing_display->strobing_on))
					result++;
	}
	else
	{
		for (strobing_display = strobing_displays->first; strobing_display; strobing_display = strobing_display->next)
			if (strobing_display->strobing_on && strobing_display->strobing_off)
				if (set_device_frequency(strobing_display->device, strobing_display->strobing_off))
					result++;
	}

	return result;
}

/****************************************************************************/

int set_frequency(int frequency)
{
	struct strobing_display *strobing_display;
	int result = 0;

	if (!frequency)
		return FALSE;

	for (strobing_display = strobing_displays->first; strobing_display; strobing_display = strobing_display->next)
		if (strobing_display->strobing_on)
			if (set_device_frequency(strobing_display->device, frequency))
				result++;

	return result;
}

/****************************************************************************/
