/* Includes *****************************************************************/

#include "vcp.h"
#include "debug.h"
#include "displays.h"
#include "i2c.h"

/* Functions ****************************************************************/

BOOL calculate_checksum(unsigned char *data, int size)
{
	int index;

	data[size] = 0;

	for (index = 0; index < size; index++)
		data[size] ^= data[index];

	return TRUE;
}

/****************************************************************************/

BOOL set_display_brightness(struct display *display, unsigned char brightness, BOOL strobed)
{
	unsigned char command_set_brightness[] = {0x6E, 0x51, 0x84, 0x03, 0x00, 0x00, 0x00, 0x00};
	unsigned char command_set_brightness_size = sizeof command_set_brightness;

	if (!display)
		return FALSE;

	if (strobed)
	{
		command_set_brightness[4] = 0xFA;
		command_set_brightness[6] = brightness;
	}
	else
	{
		command_set_brightness[4] = 0x10;
		command_set_brightness[6] = brightness * 10;
	}

	calculate_checksum(command_set_brightness, 7);

	if (!i2c_send(display, 0, 0x6E, command_set_brightness, command_set_brightness_size))
	{
		debug_printf("INFO(%s, %i, %i): BRIGHTNESS = ERROR", display->adapter->vendor, display->adapter->id, display->id);
		return FALSE;
	}

	debug_printf("INFO(%s, %i, %i): BRIGHTNESS = %i", display->adapter->vendor, display->adapter->id, display->id, brightness);
	return TRUE;
}

/****************************************************************************/

int set_brightness(struct display_list *display_list, unsigned char brightness, BOOL strobed)
{
	struct display *display;
	int result = 0;

	if (!display_list)
		return FALSE;

	for (display = display_list->first; display; display = display->next)
		if (set_display_brightness(display, brightness, strobed))
			result++;

	return result;
}

/****************************************************************************/
